@@ -1,5 +1,5 @@
 // Copyright Contributors to the OpenImageIO project.
-// SPDX-License-Identifier: BSD-3-Clause and Apache-2.0
+// SPDX-License-Identifier: Apache-2.0
 // https://github.com/AcademySoftwareFoundation/OpenImageIO
 
 
@@ -56,10 +56,8 @@
 #    define OIIO_TIFFLIB_VERSION 40003
 #elif TIFFLIB_VERSION >= 20111221
 #    define OIIO_TIFFLIB_VERSION 40000
-#elif TIFFLIB_VERSION >= 20090820
-#    define OIIO_TIFFLIB_VERSION 30900
 #else
-#    error "libtiff 3.9.0 or later is required"
+#    error "libtiff 4.0.0 or later is required"
 #endif
 // clang-format on
 
@@ -103,7 +101,8 @@ public:
     bool valid_file(Filesystem::IOProxy* ioproxy) const override;
     int supports(string_view feature) const override
     {
-        return (feature == "exif" || feature == "iptc" || feature == "ioproxy");
+        return (feature == "exif" || feature == "iptc" || feature == "ioproxy"
+                || feature == "multiimage");
         // N.B. No support for arbitrary metadata.
     }
     bool open(const std::string& name, ImageSpec& newspec) override;
@@ -210,7 +209,9 @@ private:
     // Read tags from the current directory of m_tif and fill out spec.
     // If read_meta is false, assume that m_spec already contains valid
     // metadata and should not be cleared or rewritten.
-    void readspec(bool read_meta = true);
+    // Return true if all is fine, false if something really bad happens,
+    // like we think the file is hopelessly corrupted.
+    bool readspec(bool read_meta = true);
 
     // Figure out all the photometric-related aspects of the header
     void readspec_photometric();
@@ -269,8 +270,8 @@ private:
         if (!passcount && readcount > 0) {
             return TIFFGetField(m_tif, tag, dest);
         }
-        // OIIO::debugf(" stgf %s tag %d %s datatype %d passcount %d readcount %d\n",
-        //              name, tag, type, int(TIFFFieldDataType(field)), passcount, readcount);
+        // OIIO::debugfmt(" stgf {} tag {} {} datatype {} passcount {} readcount {}\n",
+        //                name, tag, type, int(TIFFFieldDataType(field)), passcount, readcount);
         return false;
     }
 
@@ -479,7 +480,10 @@ private:
     {
         TIFFInput* self = (TIFFInput*)user_data;
         spin_lock lock(self->m_last_error_mutex);
+        OIIO_PRAGMA_WARNING_PUSH
+        OIIO_GCC_PRAGMA(GCC diagnostic ignored "-Wformat-nonliteral")
         self->m_last_error = Strutil::vsprintf(fmt, ap);
+        OIIO_PRAGMA_WARNING_POP
         return 1;
     }
 
@@ -489,7 +493,10 @@ private:
     {
         TIFFInput* self = (TIFFInput*)user_data;
         spin_lock lock(self->m_last_error_mutex);
+        OIIO_PRAGMA_WARNING_PUSH
+        OIIO_GCC_PRAGMA(GCC diagnostic ignored "-Wformat-nonliteral")
         self->m_last_error = Strutil::vsprintf(fmt, ap);
+        OIIO_PRAGMA_WARNING_POP
         return 1;
     }
 #endif
@@ -535,7 +542,10 @@ oiio_tiff_last_error()
 static void
 my_error_handler(const char* /*str*/, const char* format, va_list ap)
 {
+    OIIO_PRAGMA_WARNING_PUSH
+    OIIO_GCC_PRAGMA(GCC diagnostic ignored "-Wformat-nonliteral")
     oiio_tiff_last_error() = Strutil::vsprintf(format, ap);
+    OIIO_PRAGMA_WARNING_POP
 }
 
 
@@ -611,7 +621,10 @@ reader_mapproc(thandle_t, tdata_t*, toff_t*)
     return 0;
 }
 
-static void reader_unmapproc(thandle_t, tdata_t, toff_t) {}
+static void
+reader_unmapproc(thandle_t, tdata_t, toff_t)
+{
+}
 
 
 
@@ -821,7 +834,8 @@ TIFFInput::seek_subimage(int subimage, int miplevel)
     m_next_scanline = 0;  // next scanline we'll read
     if (subimage == m_subimage || TIFFSetDirectory(m_tif, subimage)) {
         m_subimage = subimage;
-        readspec(read_meta);
+        if (!readspec(read_meta))
+            return false;
 
         char emsg[1024];
         if (m_use_rgba_interface && !TIFFRGBAImageOK(m_tif, emsg)) {
@@ -920,7 +934,7 @@ TIFFInput::spec_dimensions(int subimage, int miplevel)
 #define ICC_PROFILE_ATTR "ICCProfile"
 
 
-void
+bool
 TIFFInput::readspec(bool read_meta)
 {
     uint32_t width = 0, height = 0, depth = 0;
@@ -1009,8 +1023,7 @@ TIFFInput::readspec(bool read_meta)
         m_spec.tile_depth  = 0;
     }
 
-    m_bitspersample = 8;
-    TIFFGetField(m_tif, TIFFTAG_BITSPERSAMPLE, &m_bitspersample);
+    TIFFGetFieldDefaulted(m_tif, TIFFTAG_BITSPERSAMPLE, &m_bitspersample);
     m_spec.attribute("oiio:BitsPerSample", (int)m_bitspersample);
 
     unsigned short sampleformat = SAMPLEFORMAT_UINT;
@@ -1154,7 +1167,8 @@ TIFFInput::readspec(bool read_meta)
                 // This extra channel is not alpha at all.  Undo any
                 // assumptions we previously made about this channel.
                 if (m_spec.alpha_channel == c) {
-                    m_spec.channelnames[c] = Strutil::sprintf("channel%d", c);
+                    m_spec.channelnames[c] = Strutil::fmt::format("channel{}",
+                                                                  c);
                     m_spec.alpha_channel   = -1;
                 }
             }
@@ -1189,7 +1203,7 @@ TIFFInput::readspec(bool read_meta)
     // assumed to be identical to what we already have in m_spec,
     // skip everything following.
     if (!read_meta)
-        return;
+        return true;
 
     short resunit = -1;
     TIFFGetField(m_tif, TIFFTAG_RESOLUTIONUNIT, &resunit);
@@ -1227,8 +1241,13 @@ TIFFInput::readspec(bool read_meta)
         m_spec.attribute(ICC_PROFILE_ATTR,
                          TypeDesc(TypeDesc::UINT8, icc_datasize), icc_buf);
         std::string errormsg;
-        decode_icc_profile(cspan<uint8_t>(icc_buf, icc_datasize), m_spec,
-                           errormsg);
+        bool ok = decode_icc_profile(cspan<uint8_t>(icc_buf, icc_datasize),
+                                     m_spec, errormsg);
+        if (!ok && OIIO::get_int_attribute("imageinput:strict")) {
+            errorfmt("Possible corrupt file, could not decode ICC profile: {}\n",
+                     errormsg);
+            return false;
+        }
     }
 
     // Search for an EXIF IFD in the TIFF file, and if found, rummage
@@ -1251,6 +1270,9 @@ TIFFInput::readspec(bool read_meta)
             // should be interpreted to be sRGB.
             if (m_spec.get_int_attribute("Exif:ColorSpace") != 0xffff)
                 m_spec.attribute("oiio:ColorSpace", "sRGB");
+            // NOTE: We must set "oiio:ColorSpace" explicitly, not call
+            // set_colorspace, or it will erase several other TIFF attribs we
+            // need to preserve.
         }
         // TIFFReadEXIFDirectory seems to do something to the internal state
         // that requires a TIFFSetDirectory to set things straight again.
@@ -1355,6 +1377,8 @@ TIFFInput::readspec(bool read_meta)
 
     if (m_testopenconfig)  // open-with-config debugging
         m_spec.attribute("oiio:DebugOpenConfig!", 42);
+
+    return true;
 }
 
 
@@ -1411,7 +1435,8 @@ TIFFInput::readspec_photometric()
             }
             // No ink names. Make it up.
             for (int i = numberofinks; i < m_spec.nchannels; ++i)
-                m_spec.channelnames.emplace_back(Strutil::sprintf("ink%d", i));
+                m_spec.channelnames.emplace_back(
+                    Strutil::fmt::format("ink{}", i));
         }
         break;
     }
@@ -1667,7 +1692,9 @@ TIFFInput::read_native_scanline(int subimage, int miplevel, int y, int /*z*/,
                                                 m_rgbadata.data(),
                                                 ORIENTATION_TOPLEFT, 0);
             if (!ok) {
-                errorfmt("Unknown error trying to read TIFF as RGBA");
+                std::string err = oiio_tiff_last_error();
+                errorfmt("Unknown error trying to read TIFF as RGBA ({})",
+                         err.size() ? err.c_str() : "unknown error");
                 return false;
             }
         }
@@ -1948,7 +1975,7 @@ TIFFInput::read_native_scanlines(int subimage, int miplevel, int ybegin,
             char* cbuf        = compressed_scratch.get() + stripidx * cbound;
             tstrip_t stripnum = (y - m_spec.y) / m_rowsperstrip;
             tsize_t csize     = TIFFReadRawStrip(m_tif, stripnum, cbuf,
-                                             tmsize_t(cbound));
+                                                 tmsize_t(cbound));
             if (csize < 0) {
                 std::string err = oiio_tiff_last_error();
                 errorfmt("TIFFRead{}Strip failed reading line y={},z={}: {}",
@@ -2048,7 +2075,9 @@ TIFFInput::read_native_tile(int subimage, int miplevel, int x, int y, int z,
         m_rgbadata.resize(m_spec.tile_pixels());
         bool ok = TIFFReadRGBATile(m_tif, x, y, m_rgbadata.data());
         if (!ok) {
-            errorfmt("Unknown error trying to read TIFF as RGBA");
+            std::string err = oiio_tiff_last_error();
+            errorfmt("Unknown error trying to read TIFF as RGBA ({})",
+                     err.size() ? err.c_str() : "unknown error");
             return false;
         }
         // Copy, and use stride magic to reverse top-to-bottom, because
